#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

#include <fstream>
#include <vector>
#include <math.h>

#define NUM_COEFF 13

struct fp {
	double mean[NUM_COEFF];
	double std[NUM_COEFF];
};

/* ! \brief Calculates the similarity matrix for given fingerprints
 *
 * Reads the fingerprints from a specified file and calculates the similarity matrix via the invers euclidean distance
 * 
 * \param argv[1]: the file containing the fingerprints (in binary format)
 * \param argv[2]: the output file in which the matrix is stored (test based)
 */

int main(int argc, char* argv[])
{
	if(argc<=2) {
		printf("You need to specify an input fingerprint file and an output file!");
		return -1;
	}


	//read fingerprint file
	FILE* fFingerprint = fopen(argv[1], "rb");

	if(fFingerprint == NULL) {
		printf("Could not open input file");
		return -1;
	}

	std::vector<fp> fingerprints;
	fp tmp;
	size_t result;
	do {
		result = fread((void*)&tmp,sizeof(fp),1,fFingerprint);
		if(result) {
			fingerprints.push_back(tmp);
		}
	} while(result);
	fclose(fFingerprint);
	printf("found %d fingerprints.\n",fingerprints.size());


	//gen the matrix
	unsigned int nF = fingerprints.size();
	double* mat = new double[nF*nF];

	for(unsigned int y=0; y<nF; y++) {
		for(unsigned int x=0; x<nF; x++) {
			double d = 0.0;
			if(y!=x) {
				//calc the distance
				fp o = fingerprints.at(x);
				fp p = fingerprints.at(y);

				double sum = 0.0;
				for(unsigned int c=0; c < NUM_COEFF; c++) {
					double mean = o.mean[c] - p.mean[c];
					double std = o.std[c] - p.std[c];

					sum += mean * mean + std * std;
				}

				d = 1.0 / (1.0 + sqrt(sum));
			}
			mat[x*nF+y] = d;
			mat[y*nF+x] = d;
		}
	}

	//write matrix
	std::ofstream mFile(argv[2]);

	for(unsigned int y=0; y<nF; y++) {
		for(unsigned int x=0; x<nF; x++) {
			mFile << mat[y*nF+x] << ",";
		}
		mFile << std::endl;
	}

	mFile.close();

	return 0;
}