var sys; //our general system handler

var dt = 10; //value of how much the screen is moved per key press

//****************** movement handling
var tx = 0;
var ty = 0;
var zoom = 1;

var w = 0;
var h = 0;

//****************** translating & scaling

function x(ox) { //transforms x coordinate
	var ts = (w - (w * zoom))/2.0; //the translate related to scaling
	var x = ox * zoom - tx + ts;
	return x;
};

function y(oy) { //transforms y coordinate
	var ts = (h - (h * zoom))/2.0; //the translate related to scaling
	var y = oy * zoom - ty + ts;
	return y;
};

function ix(ox) { //invers transform for x
	var ts = (w - (w * zoom))/2.0; //the translate related to scaling
	var x = (ox + tx - ts) / zoom ;
	return x;
};

function iy(oy) { //inverse transform for y
	var ts = (h - (h * zoom))/2.0; //the translate related to scaling
	var y = (oy + ty - ts) / zoom ;
	return y;
};

//scale values (for textwidth etc.)
function s(os) { //standard scaling based on zoom
	return Math.round(os * zoom);
};

function is(os) { //inverse scaling based on zoom
	return Math.round(os / zoom);
};

var scaleNode = false; //set to scale labels of nades on zoom as well

function setScaleNode(scaleNodesOnZoom) { //setter, just for fun
	scaleNode = scaleNodesOnZoom;
};

/************************************ 
 
			main function
 
 renders the output from our
 particle system
 
************************************/

var Renderer = function(canvas){

	//drawing variables
    var canvas = $(canvas).get(0)
    var ctx = canvas.getContext("2d");
	var gfx = arbor.Graphics(canvas)
    var particleSystem
	
	//highlighting is stored here
	var highlight = null;
	var highlightCon = null;
	
	//just some helper variables
	w = canvas.width; h = canvas.height;

    var that = {
      init:function(system){
        
        particleSystem = system;
        particleSystem.screenSize(canvas.width, canvas.height) ;
        particleSystem.screenPadding(80); // leave an extra 80px of whitespace per side
        
        //setup input handlers
        that.initMouseHandling()
		that.initKeyHandling()
      },
      
	  // draws our particle system
      redraw:function(){
	  
		//clearing
        ctx.fillStyle = "white"
        ctx.fillRect(0,0, canvas.width, canvas.height)
        
		// just for calculating the mean of all connected node positions for each node
		// edge: {source:Node, target:Node, length:#, data:{}}
		// pt1:  {x:#, y:#}  source position in screen coords
		// pt2:  {x:#, y:#}  target position in screen coords
		particleSystem.eachEdge(function(edge, pt1, pt2){
		  edge.source.data.neigMean.x += pt2.x;
		  edge.source.data.neigMean.y += pt2.y;
		  
		  edge.target.data.neigMean.x += pt1.x;
		  edge.target.data.neigMean.y += pt1.y;
		  
		  edge.source.data.neigNum += 1.0;	
		  edge.target.data.neigNum += 1.0;
		})
		
		// actually draws the edges
		// edge: {source:Node, target:Node, length:#, data:{}}
        // pt1:  {x:#, y:#}  source position in screen coords
        // pt2:  {x:#, y:#}  target position in screen coords
        particleSystem.eachEdge(function(edge, pt1, pt2){
        
          //stroking style
		  strk = "#999999";
		  if(edge.source == highlight || edge.target == highlight) strk = "#990000";
		  ctx.strokeStyle = strk;
		  
		  //line width
          ctx.lineWidth = Math.max(1,edge.data.weight*10);
		  
		  //draw the line - pay attention to the x,y transform
          ctx.beginPath();
          ctx.moveTo(x(pt1.x), y(pt1.y));
		  ctx.bezierCurveTo( x(edge.source.data.neigMean.x / edge.source.data.neigNum), y(edge.source.data.neigMean.y / edge.source.data.neigNum), x(edge.target.data.neigMean.x / edge.target.data.neigNum), y(edge.target.data.neigMean.y / edge.target.data.neigNum), x(pt2.x),y(pt2.y));
          ctx.stroke();
		  
		  //save for node highliting (we wanna point out the neighbours as well)
		  if(edge.source == highlight) highlightCon[highlightCon.length] = edge.target;
		  if(edge.target == highlight) highlightCon[highlightCon.length] = edge.source;
        })

		// draw the nodes
		// node: {mass:#, p:{x,y}, name:"", data:{}}
        // pt:   {x:#, y:#}  node position in screen coords
        particleSystem.eachNode(function(node, pt){
         
		  //define the border
		  strk = "#999999";
		  if(node == highlight) strk= "#FF0000";
		  
		  //check if neighbor, if yes, highlight
		  var con = false;
		  if(highlightCon != null)
		  for(i=0; i < highlightCon.length; i++) {
			if(highlightCon[i] == node) {
				con = true;
			}
		  }
		  if(con) strk= "#990000";
		  
		  //draw the label and the 'body'
		  ctx.lineWidth = 2; //reset line width
          var w = Math.max(20, 20+gfx.textWidth(node.data.title) ); //get the future textwidth
		  if(scaleNode) { //pay attention to the transformation
		  
			gfx.rect(x(pt.x-w/2), y(pt.y-8), s(w), s(20), s(4), {fill:node.data.color, stroke:strk, alpha:1});
			gfx.text(node.data.title, x(pt.x), y(pt.y+9), {color:"white", align:"center", font:"Arial", size:s(12)});
		  }
		  else {
		    gfx.rect(x(pt.x)-w/2, y(pt.y)-8, w, 20, 4, {fill:node.data.color, stroke:strk, alpha:1});
            gfx.text(node.data.title, x(pt.x), y(pt.y)+9, {color:"white", align:"center", font:"Arial", size:12});
		  }
		  
		  //reset means and such
		  node.data.neigNum = 0.0;
		  node.data.neigMean.x = 0.0;
		  node.data.neigMean.y = 0.0;
        })    			
      },
      
      initMouseHandling:function() {
        var dragged = null; // to prevent nonsense drag and drop
		
        var handler = {
          clicked:function(e){
            var pos = $(canvas).offset();
            _mouseP = arbor.Point(ix(e.pageX-pos.left), iy(e.pageY-pos.top)); //calculate the mousepointer on the particle system --> inverse transform
            dragged = particleSystem.nearest(_mouseP); //find the nearest node

            if (dragged && dragged.node !== null){ //lets drag

				dragged.node.fixed = true //disable physics on the node
				highlight = dragged.node; //highlight it
				highlightCon = new Array();

				setSong(dragged.node.data.file); //adapt the player to the selected song
				$('#mp3_title').html("<b>"+dragged.node.data.title+"</b>"); 
				$('#mp3_artist').html("<b>"+dragged.node.data.artist+"</b>"); 
				$('#mp3_album').html("<b>"+dragged.node.data.album+" ("+dragged.node.data.year+")</b>"); 
				$('#player').show();

            }

			//bind stuff
            $(canvas).bind('mousemove', handler.dragged);
            $(window).bind('mouseup', handler.dropped);
			
            return false;
          },
		  
		  //when we hold the mousebutton and move it
          dragged:function(e){
            var pos = $(canvas).offset();
            var s = arbor.Point(ix(e.pageX-pos.left), iy(e.pageY-pos.top));

			//set the postion
            if (dragged && dragged.node !== null){
              var p = particleSystem.fromScreen(s);
              dragged.node.p = p;
            }

            return false;
          },

		  //when we release the mouse button
          dropped:function(e){
			//error states
            if (dragged===null || dragged.node===undefined) return;
            if (dragged.node !== null) dragged.node.fixed = false;
			
            dragged.node.tempMass = 1000; //reajust the graph
            dragged = null;
			
			//unbind
            $(canvas).unbind('mousemove', handler.dragged);
            $(window).unbind('mouseup', handler.dropped);
            _mouseP = null;
            return false;
          },
		  
		  //for zooming
		  scroll:function(event, delta, deltaX, deltaY){
			//determin which kind of zoom (out/in)
			var s = 1; 
			if(delta > 0) { //up
				s = 1.1;
			}
			else { //down
				s = 1.0/1.1;
			}
			
			//set and redraw
			zoom = zoom * s;
			that.redraw();
          }
        }
        
        // start listening
        $(canvas).mousedown(handler.clicked);
		//scroll
		$(canvas).bind('mousewheel', handler.scroll); //uses jquery plugin

      },
      
	  //for keyboard translation
	  initKeyHandling:function() {
	  
		$(document).keydown(function(event) {
			if(event.which == 39) { //right arrow
				tx += dt; 
			}
			if(event.which == 37) { //left arrow
				tx -= dt; 
			}
			if(event.which == 38) { //up arrow
				ty -= dt; 
			}
			if(event.which == 40) { //down arrow
				ty += dt; 
			}
			//that.redraw(); //breaks the system, because redraw is called too often
		});
	  }
    }
    return that;
  }    
