function [msg] =  addFingerprint( database, soundFile)
    addpath('voicebox/');

    %global parameters as stated in the paper
    samplesPerWindow = 2048;
    numC = 13; %number of mfcc coefficients
    endings = {'mp3' 'wav'};

    %pre alloc
    fingerprints = {};
    
    
    if(isdir(soundFile))
        fingerprints = searchDirForFiles(soundFile,endings);
    else
        if exist(soundFile, 'file')
            fingerprints{1,1} = soundFile;
        end
    end
    
    countGen = 0;
    for i=1:size(fingerprints,2);
        %audio processing starts here
        sf = fingerprints{1,i};
        
        [d,Fs] = audioread(sf);
        d = mean(d,2); %to mono

        c = melcepst(d,Fs,'M',numC,samplesPerWindow);

        fingerprints{2,i} = mean(c,1);
        fingerprints{3,i} = std(c,1,1);
        
        %this is our fingerprint
        
        countGen = countGen + 1;
        
        disp(['Generated fingerprint for ' sf]);
        clear c; clear d;
    end
    
    msg = ['Successfully generated ' num2str(countGen) ' fingerprints'];
    
    if exist(database, 'file')
        db = load(database,'-mat');
        
        numDbEle = size(fingerprints,2);
        for i=1:numDbEle
            ex = 0;
            for j=1:size(db.fingerprints,2) %check if entry already exists
                if strcmp(fingerprints{1,i},db.fingerprints{1,j})
                   ex = j; 
                end
            end
            
            if ex == 0 %if entry doenst exists yet
                db.fingerprints{1,numDbEle+1} = fingerprints{1,i};
                db.fingerprints{2,numDbEle+1} = fingerprints{2,i};
                db.fingerprints{3,numDbEle+1} = fingerprints{3,i};
            else %calling it 3 times because it wont work with :
                db.fingerprints{1,ex} = fingerprints{1,i};
                db.fingerprints{2,ex} = fingerprints{2,i};
                db.fingerprints{3,ex} = fingerprints{3,i};
            end
        end
        
        fingerprints = db.fingerprints;
        save(database,'fingerprints');
        
        msg = [msg ' and added to the database ' database ' with currently ' num2str(size(fingerprints,2)) ' entries '];
    else
        save(database,'fingerprints');
        msg = [msg ' and generated database ' database];
    end
end