function [ msg ] = genSimilarityMatrix( fingerprintDB, outputMatrix, outputList)
%GENSIMILARITYMATRIX Summary of this function goes here
%   Detailed explanation goes here

    if nargin < 2 || ~exist(outputMatrix, 'file')
        outputMatrix = 'simMatrix.txt';
    end
    if nargin < 3 || ~exist(outputList, 'file')
        outputList = 'fileList.txt';
    end

    if exist(fingerprintDB, 'file')
        db = load(fingerprintDB,'-mat');
        fp = db.fingerprints;
        
        listFile = fopen(outputList,'w');
        matrixFile = fopen(outputMatrix,'w');

        %do the calc
        numFp = size(fp,2);
        
        simM = zeros(numFp,numFp);
        
        st = 2;
        for i = 1:numFp
            for j=st:numFp
                
                meanDiff = fp{2,i} - fp{2,j};
                stdDiff = fp{3,i} - fp{3,j};
                
                meanDiff = meanDiff .^ 2;
                stdDiff = stdDiff .^ 2;
                
                w = 1 / (1 + sqrt(sum(meanDiff)+sum(stdDiff))); % inverse euclidean
                
                simM(i,j) = w;
                simM(j,i) = w;
            end
          
            st = st + 1;
        end
        
        %file saving loop
        for i = 1:numFp
            for j=1:numFp-1
                fprintf(matrixFile,'%f,',simM(i,j));
            end
            fprintf(matrixFile,'%f\r\n',simM(i,numFp));
            
            %list entry
            tags = id3parse(fp{1,i});
            Title = ' '; if isfield(tags,'Title') Title = tags.Title; end
            Artist = ' '; if isfield(tags,'Artist') Artist = tags.Artist; end
            Album = ' '; if isfield(tags,'Album') Album = tags.Album; end
            Recording_Year = ' '; if isfield(tags,'Recording_Year') Recording_Year = tags.Recording_Year; end
            fprintf(listFile,'%s;',fp{1,i});
            fprintf(listFile,'%s;',Title);
            fprintf(listFile,'%s;',Artist);
            fprintf(listFile,'%s;',Album);
            fprintf(listFile,'%s;',Recording_Year);
            fprintf(listFile,'\r\n');
        end
        
        fclose(listFile);
        fclose(matrixFile);
        
        msg = ['Successfully created similarity matrix and saved it @ ' outputMatrix]; 
    else
       msg = 'Error: Fingerprint database not found!'; 
    end
end

